
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

#include <stdio.h>
#include <string.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/xv/xlogo11>
#include <X11/xv/xvCore.h>
#include <X11/xv/xvFont.h>
#include <X11/xv/xvPopupShell.h>
#include <X11/xv/xvPopupMenu.h>
#include <X11/Xaw3d/SimpleMenu.h>
#include <X11/Xaw3d/SmeBSB.h>
#include <X11/Xaw3d/SmeLine.h>

// ******************************************************

xvPopupMenu::xvPopupMenu()
{
}

// -----------------------------------------------

xvPopupMenu::xvPopupMenu(char *name, char *header, int argc, char **argv, 
			XtCallbackProc proclist[], xvCore & parent)
{
	init(name, header, argc, argv, proclist, parent);
}

// -----------------------------------------------

xvPopupMenu::~xvPopupMenu()
{
}

// -----------------------------------------------

int xvPopupMenu::findentry(char *name)
{
	int i=-1;
	for(int j=0; j<numentries; j++)
		if (strcmp(listentry[j], name)==0) {
			i=j;
			break;
		}

	return(i);
}

// -----------------------------------------------

void xvPopupMenu::EntryResource(String name, String resname, XtArgVal res)
{
	Widget xw=GetWidget(name);
	if (xw !=NULL)
		XtVaSetValues(xw, resname, res, NULL);
}

// -----------------------------------------------

void xvPopupMenu::MenuEntry(String name, String newname)
{
	Widget xw=GetWidget(name);
	if (xw != NULL)
		XtVaSetValues(xw, XtNlabel, (XtArgVal) newname, NULL);
}

// -----------------------------------------------

void xvPopupMenu::CheckEntry(char *name)
{
	Widget xw=GetWidget(name);
	if (xw != NULL)
		XtVaSetValues(xw, XtNleftBitmap, mark, NULL);
				
}

// -----------------------------------------------

void xvPopupMenu::UnCheckEntry(char *name)
{
	Widget xw=GetWidget(name);
	if (xw != NULL)
		XtVaSetValues(xw, XtNleftBitmap, None, NULL);
}

// -----------------------------------------------

Widget xvPopupMenu::GetWidget(char *name)
{

	int i=findentry(name);

	if (i ==-1)
		return(NULL);
	else
		return(WidgetList[i]);
		
}

// -----------------------------------------------

void xvPopupMenu::init(char *name, char *header, int argc, char **argv, 
				XtCallbackProc proclist[],  xvCore & parent)
{
	char mtran[]="#augment\n\
				<Motion>: highlight()";

	mtranslations=XtParseTranslationTable(mtran);

	xvPopupShell::init(name, parent, simpleMenuWidgetClass, header, mtranslations);

	Sep=XtVaCreateManagedWidget(NULL, smeLineObjectClass, ID(),
		"label", NULL, "leftMargin", 20, NULL);

        numentries=argc;
	MyParent=parent.ID();
	bfont.init(MyParent, NULL);

	for (int i=0; i<argc; i++) { 
	  listentry[i]=argv[i];
          if (strcmp(argv[i],"---") == 0) 
            WidgetList[i]=XtVaCreateManagedWidget(NULL, smeLineObjectClass,
	      ID(), "label", NULL, "leftMargin", 20, NULL);
          else
            WidgetList[i]=XtVaCreateManagedWidget(NULL, smeBSBObjectClass,
	      ID(), "label", argv[i], "leftMargin", 20, NULL);

	  XtAddCallback(WidgetList[i], XtNcallback, proclist[i], argv[i]);
        }
	mark=XCreateBitmapFromData(XtDisplay(parent.ID()),
		RootWindowOfScreen(XtScreen(parent.ID())),
		xlogo11_bits, xlogo11_width, xlogo11_height);

}  	

// ******************************************************

void xvPopupMenu::AllResource(String resname, XtArgVal res)
{
	for (int i=0; i<numentries; i++)
		XtVaSetValues(WidgetList[i], resname, res, NULL);
}

// -----------------------------------------------

void xvPopupMenu::ChangeFont(String fontname)
{
	if (fontname != NULL) {
		bfont.FLoadFont(fontname);
		AllResource(XtNfont, (XtArgVal) bfont.FontRef());
	}
}

// -----------------------------------------------------------------
