
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

/* =============================================
	xvis user interface builder for Athena widgets using
	xv class library.

	Modules Segment

	V1.0BETA  - last edited 25/06/96
    ============================================= */

// ---------------------------------------------
// Include file declarations

#include "xvis.h"
#include <string.h>
#include <stdio.h>

extern int CurrentModule;
extern Boolean xlocked;
extern int readBlock(String buffer, int num, FILE *fp);
extern int fileExist(String name);

// ---------------------------------------------
// Local data

Boolean locsaveas;

// ---------------------------------------------
// local forward function declares

int saveCurrentModule();
int loadCurrentModule(String name);

// --------------------------------------------
// Callbacks


void module_ok(Widget aw, XtPointer client, XtPointer call)
{
	char *temp=(char *) client;
	int which;
	sscanf(temp, "%d", &which);

	XtVaGetValues(module[which].editor->xvEditField::ID(), XtNstring, &temp, NULL);
	strcpy(module[which].edit_buffer, temp);
	XtUnmapWidget(module[which].editor->xvWindow::GetParent());
	module[which].changed=TRUE;
}

void module_cancel(Widget aw, XtPointer client, XtPointer call)
{
	char *temp=(char *) client;
	int which;
	sscanf(temp, "%d", &which);
	XtUnmapWidget(module[which].editor->xvWindow::GetParent());
}

void mod_ow_yes(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
	int err=saveCurrentModule();
	xlocked=FALSE;
}

void mod_ow_no(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
	xlocked=FALSE;
}

void mod_ok_load(Widget aw, XtPointer client, XtPointer call)
{
	strcpy(provpath, xvisSelFile.SelectedFile(propeditbuffer));
	xvisSelectFile.RemoveGrab();
	xvisSelectFile.Close();

	if (fileExist(provpath)==-1) {
		alert.init("File Does Not Exist", xvisControl, 
			"The specified file does no exist",
			XVMSGSIMPLEFATAL, "red",
			mod_ow_no, mod_ow_no);
		alert.Open();
	}
	else {
		int err=loadCurrentModule(provpath);
		updateProjectListWindow();
	}
}

void mod_ok_save(Widget aw, XtPointer client, XtPointer call)
{
	Boolean changen=FALSE;

	strcpy(provpath, xvisSelFile.SelectedFile(propeditbuffer));
	changen=(strcmp(provpath, module[CurrentModule].name)==0);
	strcpy(module[CurrentModule].path, propeditbuffer);
	strcpy(module[CurrentModule].name, provpath);
	xvisSelectFile.RemoveGrab();
	xvisSelectFile.Close();
	updateProjectListWindow();

	if ((fileExist(propeditbuffer)!=-1) && (! changen)) {
		alert.init("File Exists", xvisControl, 
			"The specified file exists.\n Do you want to Overwrite?",
			XVMSGSIMPLECHOICE, "green",
			mod_ow_yes, mod_ow_no);
		alert.Open();
	}
	else {
		int err=saveCurrentModule();
		xlocked=FALSE;
	}
}

//---------------------------------------------
// Local Stuff

void makeNewModule()
{
	int pe=findFirstModuleEntry();
	if (pe!=-1) {
		sprintf(module[pe].client, "%d", pe);
		module[pe].in_use=TRUE;
		sprintf(module[pe].name, "Noname%d.cc", pe);
		sprintf(module[pe].path, "Noname%d.cc", pe);
		module[pe].edit_buffer=new char[MAXEDITBUFFERSIZE];
		module[pe].edit_buffer[0]='\0';
		module[pe].editor=new xvEditWindow(xvisContext, module[pe].name, 
		XawtextEdit, module[pe].edit_buffer, 600, 400, 
		module_ok, module_cancel, module[pe].client);	
		module[pe].editor->ChangeFont("*times-*-r-*-140*");		
		module[pe].editor->SetWindowResource(XtNiconPixmap, 
			module[pe].editor->xvEditField::SetPixmapInfo(xvis_xpm, NULL));
		module[pe].editor->xvEditField::SetBackground("LemonChiffon");

		module[pe].editor->Open();
		module[pe].active=TRUE;		
		updateProjectListWindow();
	}

}

// ----------------------------------------------

void loadModule()
{
	
	buildSelectFileWindow("Load Module...", mod_ok_load, ".cc", TRUE);
	xvisSelectFile.Open();
	xvisSelFile.UpdateFileBox(".cc");
	xvisSelectFile.AddGrab(FALSE, FALSE);	
}

// ----------------------------------------------		

void saveModule(Boolean saveas)
{
	locsaveas=saveas;
	if (CurrentModule== -1) return;

	if ((locsaveas) || (module[CurrentModule].first)) {
		buildSelectFileWindow("Save Module...", mod_ok_save, module[CurrentModule].name);
		xvisSelectFile.Open();
		xvisSelFile.UpdateFileBox(module[CurrentModule].name);
		xvisSelectFile.AddGrab(FALSE, FALSE);	
	}
	else {
		int err=saveCurrentModule();
		xlocked=FALSE;
	}
}

// ----------------------------------------------		

int saveCurrentModule()
{
	FILE *fp;
	module[CurrentModule].editor->xvWindow::SetWindowResource(XtNtitle, 
		(XtArgVal) module[CurrentModule].name);
	fp=fopen(module[CurrentModule].path, "w");

	if (fp!=NULL) {
		fprintf(fp, "%s", module[CurrentModule].edit_buffer);
		fclose(fp);
		module[CurrentModule].changed=FALSE;
		module[CurrentModule].first=FALSE;
	}
	return(0);
}

// ----------------------------------------------

char *basename(char *name)
{
	char stemp[90];
	int i=strlen(name);
	int j=i;
	while ( (name[j] != '/') && (j>-1))
		j--;
	if (j<0)
		return(name);
	else {
		for (int k=j+1; k<=i; k++)
			stemp[k-(j+1)]=name[k];
		return(stemp);
	}
}

int loadCurrentModule(String name)
{
	FILE *fp;

	char *buffer=new char[MAXEDITBUFFERSIZE];

	int ne=findFirstModuleEntry();
	int len;
	buffer[0]='\0';
	if (ne==-1) return(-1);

	fp=fopen(name, "r");
	if (fp!=NULL) {
		sprintf(module[ne].client, "%d", ne);
		module[ne].in_use=TRUE;
		strcpy(module[ne].name, basename(name));
		strcpy(module[ne].path, name);
		module[ne].edit_buffer=new char[MAXEDITBUFFERSIZE];
		module[ne].edit_buffer[0]='\0';
	
		readBlock(buffer, MAXEDITBUFFERSIZE, fp);
		strcpy(module[ne].edit_buffer, buffer);
		fclose(fp);

		module[ne].editor=new xvEditWindow(xvisContext, module[ne].name, 
			XawtextEdit, module[ne].edit_buffer, 600, 400, 
			module_ok, module_cancel, module[ne].client);	
		module[ne].editor->ChangeFont("*times-*-r-*-140*");		
		module[ne].editor->SetWindowResource(XtNiconPixmap, 
			module[ne].editor->xvEditField::SetPixmapInfo(xvis_xpm, NULL));
		module[ne].editor->xvEditField::SetBackground("LemonChiffon");
		module[ne].first=FALSE;
	}
	delete buffer;
	updateProjectListWindow();
	return(0);
}

// -------------------------------------------------

		